/**
 * Created by Richard Farrell on 22/04/2017.
 */

// sprite object with built in retroFont object for easy text rendering
RetroText = function(game, font, wChar, hChar, xPos, yPos, characterSet, maxWidthPx) {

    this.game = game;

    this.wChar = wChar;
    this.hChar = hChar;
    this.maxWidthPx = (typeof(maxWidthPx) === 'number' ? maxWidthPx : false);
    this.charSet = (typeof(characterSet) === 'string' ? characterSet : 'abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ 0123456789!"%^&*)(_+-=][}{#~;:@\'<>,.?/\\|');

    this.retroFontObj = this.game.add.retroFont(
        font,           // key
        this.wChar,     // char width
        this.hChar,     // char height
        this.charSet    // the characterSet to use
    );

    Phaser.Sprite.call(this, this.game, xPos, yPos, this.retroFontObj);
    this.game.add.existing(this);

    this.events.onDestroy.add(function() {
        this.retroFontObj.destroy();
    }, this);
};
RetroText.prototype = Object.create(Phaser.Sprite.prototype);
RetroText.prototype.constructor = RetroText;
RetroText.prototype.setText = function(str, alignment) {

    if (typeof(str != 'string')) {
        str = str.toString();
    }

    // process str to make sure it fits within the maxWidth pixels
    if (this.maxWidthPx) {
        var cw = this.wChar;
        var lw = this.maxWidthPx;
        var px = 0;
        var lastSpace = false;
        var rows = 1;
        for (var i = 0, len = str.length; i < len; i++) {
            if (str[i] == "\n") {
                px = 0;
                rows++;
            } else if (px + cw > lw) {
                if (lastSpace) {
                    str = str.slice(0, lastSpace) + "\n" + str.slice(lastSpace + 1);
                    lastSpace = false;
                } else {
                    str = str.slice(0, i) + "\n" + str.slice(i);
                }
                px = 0;
                rows++;
                len++;  // because you've added a carriage return!
            } else {
                px += cw;
                if (str[i] == ' ') {
                    lastSpace = i;
                }
            }
        }
    }

    // render string to retroFont object used by sprite
    this.retroFontObj.setText(
        str,                // content
        true,               // multi-line
        0,                  // char spacing
        1,                  // line spacing
        (typeof(alignment) === 'string' ? alignment : Phaser.RetroFont.ALIGN_LEFT),     // alignment
        true                // allow lower case
    );
};
RetroText.prototype.clear = function() {

    this.retroFontObj.text = '';
};

MouseCursor = function(game, spritesheet, defaultFrame) {

    this.game = game;
    Phaser.Sprite.call(this, this.game, 0, 0, spritesheet, (defaultFrame ? defaultFrame : 0));
    this.game.add.existing(this);

    this.wasLeftDown = false;
    this.isLeftDown = false;
};
MouseCursor.prototype = Object.create(Phaser.Sprite.prototype);
MouseCursor.prototype.constructor = MouseCursor;
MouseCursor.prototype.update = function() {

//    this.bringToTop(); // might need to enable this if lots of sprites are added during game play
//    this.x = Math.round(this.game.input.mousePointer.x);
//    this.y = Math.round(this.game.input.mousePointer.y);
    this.x = Math.round(this.game.camera.x + this.game.input.activePointer.x);
    this.y = Math.round(this.game.camera.y + this.game.input.activePointer.y);

    this.wasLeftDown = this.isLeftDown;
//    this.isLeftDown = this.game.input.activePointer.leftButton.isDown;
    this.isLeftDown = this.game.input.activePointer.isDown;
};
MouseCursor.prototype.isLeftPressed = function() {
    return (this.isLeftDown && !this.wasLeftDown);
};
MouseCursor.prototype.isLeftReleased = function() {
    return (!this.isLeftDown && this.wasLeftDown);
};
MouseCursor.prototype.isLeftHeld = function() {
    return (this.isLeftDown && this.wasLeftDown);
};
MouseCursor.prototype.isLeftUp = function() {
    return (!this.isLeftDown && !this.wasLeftDown);
};
// get the mouse screen coords
MouseCursor.prototype.getXY = function() {
    return {
        x: Math.round(this.game.input.activePointer.x),
        y: Math.round(this.game.input.activePointer.y)
    };
};
// get the mouse world coords
MouseCursor.prototype.getWorldXY = function() {
    return {
        x: this.x,
        y: this.y
    };
};

ScreenButtons = function(game) {

    this.fullscreenButton = game.add.button(
        game.camera.width - 10,
        game.camera.height - 10,
        'fullscreen-button',
        function() {
            if (game.scale.isFullScreen) {
                game.scale.stopFullScreen();
            } else {
                game.scale.startFullScreen(false);
            }
        },
        this,
        2, 1, 0, 1
    );
    this.fullscreenButton.fixedToCamera = true;
    this.fullscreenButton.alpha = 0.75;

    this.minusButton = game.add.button(
        game.camera.width - 20,
        game.camera.height - 10,
        'minus-button',
        function() {
            decreaseIntegerScale();
        },
        this,
        2, 1, 0, 1
    );
    this.minusButton.fixedToCamera = true;
    this.minusButton.alpha = 0.75;

    this.plusButton = game.add.button(
        game.camera.width - 30,
        game.camera.height - 10,
        'plus-button',
        function() {
            increaseIntegerScale();
        },
        this,
        2, 1, 0, 1
    );
    this.plusButton.fixedToCamera = true;
    this.plusButton.alpha = 0.75;

    this.bringToTop();

    if (game.scale.isFullScreen) {
        this.hide();
    }

    game.scale.onFullScreenChange.add(function(e){
        if (game.scale.isFullScreen) {
            hideBrowserPointer();
            this.hide();
        } else {
            showBrowserPointer();
            this.show();
        }
    }, this);
};
ScreenButtons.prototype.hide = function(force) {
    if (game.scale.isFullScreen || force === true) {
        this.fullscreenButton.visible = false;
        this.fullscreenButton.inputEnabled = false;
        this.minusButton.visible = false;
        this.minusButton.inputEnabled = false;
        this.plusButton.visible = false;
        this.plusButton.inputEnabled = false;
    }
};
ScreenButtons.prototype.show = function(force) {
    if ((!game.scale.isFullScreen) || force === true) {
        this.fullscreenButton.visible = true;
        this.fullscreenButton.inputEnabled = true;
        this.minusButton.visible = true;
        this.minusButton.inputEnabled = true;
        this.plusButton.visible = true;
        this.plusButton.inputEnabled = true;
        this.bringToTop();
    }
};
ScreenButtons.prototype.bringToTop = function() {
    this.fullscreenButton.bringToTop();
    this.plusButton.bringToTop();
    this.minusButton.bringToTop();
};
ScreenButtons.prototype.destroy = function() {

    this.fullscreenButton.destroy();
    this.minusButton.destroy();
    this.plusButton.destroy();
};

ScreenFX = function(game) {

    if (!game.cache.checkKey(Phaser.Cache.BITMAPDATA, 'white-square-texture')) {
        let bmd = game.add.bitmapData(32,32);
        bmd.ctx.beginPath();
        bmd.ctx.rect(0,0,32,32);
        bmd.ctx.fillStyle = '#ffffff';
        bmd.ctx.fill();
        game.cache.addBitmapData('white-square-texture', bmd);
    }

    Phaser.Sprite.call(this, game, -64, -64, game.cache.getBitmapData('white-square-texture'));
    game.add.existing(this);
    this.fixedToCamera = true;
    this.width = game.camera.width + 128;
    this.height = game.camera.height + 128;
    this.visible = false;
//    this.tint = 0x0000ff;
//    this.alpha = 0.5;

    this.isRunning = false;
    this.tween = null;
    this.game = game;
};
ScreenFX.prototype = Object.create(Phaser.Sprite.prototype);
ScreenFX.prototype.constructor = ScreenFX;
ScreenFX.prototype.reset = function() {

    this.isRunning = false;
    this.tint = 0xE5E5E5;
    this.visible = false;
    if (this.tween != null) {
        this.tween = null;
    }
};
ScreenFX.prototype.fadeOut = function(duration, tint, resetOnComplete) {

    if (this.isRunning) {
        this.reset();
    }

    this.isRunning = true;
    this.tint = tint;
    this.alpha = 0;
    this.visible = true;
    this.bringToTop();

    this.tween = this.game.add.tween(this);
    this.tween.to( { alpha: 1.0 }, duration, Phaser.Easing.Linear.None);
    let that = this;
    this.tween.onComplete.add(function() {
        that.isRunning = false;
        if (resetOnComplete === true) {
            that.reset();
        }
    }, this);
    this.tween.start();
};
ScreenFX.prototype.update = function() {
    if (this.isRunning) {
        this.bringToTop();
    }
};

Level = function(game) {

    ///////////////////////////
    // LEVEL VARS + OBJ REFS //
    ///////////////////////////

    this.isLoaded = false;
    this.isPaused = false;
    this.game = game;

    //////////////////////////////////////////
    // SETUP COLLISION GROUPS               //
    //////////////////////////////////////////

    this.sprites = game.add.group();
};
Level.prototype.load = function() {

    ///////////////////////////
    // LEVEL VARS + OBJ REFS //
    ///////////////////////////

    this.isLoaded = true;
    this.finish = null;         // finishing line object
    this.horsesFinished = 0;    // also used to record position
    this.raceFinished = false;

    ////////////////////////////////
    // LOAD DATA FROM URL //
    ////////////////////////

    // counts the screen reset, to allow for infinite scrolling
    this.numOfScreens = 20;
    this.screenCount = 0;

    let img = {
        finishing_line: null,
        width: 80,
        height: 9
    };
    img.finishing_line = img.width - 13;

    // generate our tilemap data from the pixel data
    var data = '';
    for (var y = 0 ; y < img.height ; y++) {
        for (var x = 0 ; x < img.width ; x++) {

            // function for finding hex colour value @ pixel coordinate
            var findHex = function(img, x, y) {

                if (x < 0 || y < 0 || x > img.width-1 || y > img.height-1) {
                    return null;
                }

                // calculate the r-channel position of pixel[x][y]
                var ch = (x + y * img.width) * 4;

                // get the channel values (ignore alpha channel)
                var r = pxd.data[ch];
                var g = pxd.data[ch+1];
                var b = pxd.data[ch+2];

                // get hex value for identification
                return Color.rgb2hexString(r, g, b);
            };

            switch(y) {
                case 0:
                case 1:
                    data += 0;
                    break;
                case 2:
                case 3:
                case 7:
                case 8:
                    data += 1;
                    break;
                case 4:
                case 5:
                case 6:
                    data += 2;
                    break;
                default:
                    data += -1;
                    break;
            }

            // if not end of line, add a comma
            if (x < img.width - 1) {
                data += ',';
            }
        }

        // if end of line add a linebreak
        if (y < img.height) {
            data += "\n";
        }
    }

    // add tilemap data to game cache
    game.cache.addTilemap('level-tilemap', null, data, Phaser.Tilemap.CSV);
    this.tileMap = game.add.tilemap('level-tilemap', 16, 16);
    this.tileMap.addTilesetImage('tileset');
//    this.tileMap.setCollisionBetween(2, 3);
    this.layer0 = this.tileMap.createLayer(0);
    this.layer0.sendToBack();
    this.layer0.resizeWorld();

    //////////////////
    // LOAD SPRITES //
    //////////////////

    // build bars and finish line above track
    for (let x = 0 ; x < img.width ; x++) {

        // add top bar
        let spr = game.add.sprite(
            x * 16,
            4 * 16 + 3,
            (x == img.finishing_line ? 'finish' : 'bar')
        );
        spr.info = { name: (x == img.finishing_line ? 'finish' : 'bar') };
        if (x == img.finishing_line) { this.finish = spr; }
        spr.anchor.set(0, 1);

        this.sprites.add(spr);

        // add bottom bar
        spr = game.add.sprite(
            x * 16,
            7 * 16 + 4,
            'bar'
        );
        spr.info = { name: 'bar' };
        spr.anchor.set(0, 1);

        this.sprites.add(spr);
    }

    // Load the player horse

    let horse = new Horse(game, this, 32, 100, true);
//    horse.changeLane(1);
    this.sprites.add(horse);
    this.playerHorse = horse;

    // Load some AI horses
    this.sprites.add(new Horse(game, this, 35, 100 + 10));
    this.sprites.add(new Horse(game, this, 30, 100 - 10));
    this.sprites.add(new Horse(game, this, 38, 100 - 20));

    this.sprites.add(new Horse(game, this, 10, 98 + 10));
    this.sprites.add(new Horse(game, this, 6, 98 + 0));
    this.sprites.add(new Horse(game, this, 14, 98 - 10));
    this.sprites.add(new Horse(game, this, 9, 98 - 20));

    this.sprites.add(new Horse(game, this, 55, 99 + 10));
    this.sprites.add(new Horse(game, this, 58, 99 + 0));
    this.sprites.add(new Horse(game, this, 61, 99 - 10));
    this.sprites.add(new Horse(game, this, 54, 99 - 20));

    // just in case pause is stopping the depth sort
    this.sprites.sort('y', Phaser.Group.SORT_ASCENDING);

    this.sfx_gallop = game.add.audio('gallop', 0.1, false);

    this.sfx_gallop.onStop.add(function(){
//        log("onStop");
        if (this.raceFinished) {
//            log("play fadeOut");
            this.sfx_gallop_fadeOut.play();
        } else {
//            log("play normal");
            this.sfx_gallop.play();
        }
    }, this);

    this.sfx_gallop_fadeOut = game.add.audio('gallop-fadeOut', 1, false);

//    this.sfx_gallop_fadeOut.onPlay.add(function(){
//        this.sfx_gallop.stop();
//        this.sfx_gallop.stoppedFlag = true;
//    }, this);
};
Level.prototype.restart = function() {

    this.horsesFinished = 0;    // also used to record position
    this.raceFinished = false;

    // counts the screen reset, to allow for infinite scrolling
    this.screenCount = 0;

    // destroy and rebuild all sprites

    this.finish = null;
    this.playerHorse = null;
    this.sprites.removeAll(true);

    let img = {
        finishing_line: null,
        width: 80,
        height: 9
    };
    img.finishing_line = img.width - 13;

    // build bars and finish line above track
    for (let x = 0 ; x < img.width ; x++) {

        // add top bar
        let spr = game.add.sprite(
            x * 16,
            4 * 16 + 3,
            (x == img.finishing_line ? 'finish' : 'bar')
        );
        spr.info = { name: (x == img.finishing_line ? 'finish' : 'bar') };
        if (x == img.finishing_line) { this.finish = spr; }
        spr.anchor.set(0, 1);

        this.sprites.add(spr);

        // add bottom bar
        spr = game.add.sprite(
            x * 16,
            7 * 16 + 4,
            'bar'
        );
        spr.info = { name: 'bar' };
        spr.anchor.set(0, 1);

        this.sprites.add(spr);
    }

    // Load the player horse
    let horse = new Horse(game, this, 32, 100, true);
//    horse.changeLane(1);
    this.sprites.add(horse);
    this.playerHorse = horse;

    // Load some AI horses
    this.sprites.add(new Horse(game, this, 35, 100 + 10));
    this.sprites.add(new Horse(game, this, 30, 100 - 10));
    this.sprites.add(new Horse(game, this, 38, 100 - 20));

    this.sprites.add(new Horse(game, this, 10, 98 + 10));
    this.sprites.add(new Horse(game, this, 6, 98 + 0));
    this.sprites.add(new Horse(game, this, 14, 98 - 10));
    this.sprites.add(new Horse(game, this, 9, 98 - 20));

    this.sprites.add(new Horse(game, this, 55, 99 + 10));
    this.sprites.add(new Horse(game, this, 58, 99 + 0));
    this.sprites.add(new Horse(game, this, 61, 99 - 10));
    this.sprites.add(new Horse(game, this, 54, 99 - 20));

    // just in case pause is stopping the depth sort
    this.sprites.sort('y', Phaser.Group.SORT_ASCENDING);
};
Level.prototype.destroy = function() {
    this.unload();
};
Level.prototype.pause = function() {

    if (!this.isPaused) {
//        game.paused = true;
        this.isPaused = true;
//        game.physics.arcade.isPaused = true;
        for (let i = 0 ; i < this.sprites.children.length ; i++) {
            if (typeof(this.sprites.children[i].pause) !== "undefined") {
                this.sprites.children[i].pause();
            }
        }
        this.sfx_gallop.pause();
    }
};
Level.prototype.resume = function() {

    if (this.isPaused) {
//        game.paused = false;
        this.isPaused = false;
//        game.physics.arcade.isPaused = false;
        for (let i = 0 ; i < this.sprites.children.length ; i++) {
            if (typeof(this.sprites.children[i].resume) !== "undefined") {
                this.sprites.children[i].resume();
            }
        }
        if (!this.sfx_gallop.isPlaying) {
            this.sfx_gallop.fadeIn(1000, false);
        }
        this.sfx_gallop.resume();
    }
};
Level.prototype.pauseToggle = function() {
    if (this.isPaused) {
        this.resume();
    } else {
        this.pause();
    }
};
Level.prototype.updateHorseAI = function() {

    for (let i = 0 ; i < this.sprites.children.length ; i++) {
        let horse = this.sprites.children[i];
        if (horse.info.name == 'horse' && !horse.isPlayer) {
            horse.updateAI();
        }
    }

};
Level.prototype.update = function() {

    if (this.isPaused) { return; }

    //////////////////////////////
    // DEPTH SORT GRAPHICS      //
    //////////////////////////////

    this.sprites.sort('y', Phaser.Group.SORT_ASCENDING);

    this.game.physics.arcade.collide(this.sprites, this.sprites,
        function(spr1, spr2) {
            // collisionHandler
//            log('collision');
        },
        function(spr1, spr2) {
            // processHandler
            return true;
        },
        this
    );

    if (this.screenCount < this.numOfScreens && this.playerHorse.x >= 700) {
        for (let i = 0 ; i < this.sprites.children.length ; i++) {
            if (this.sprites.children[i].info.name == 'horse') {
                this.sprites.children[i].x -= 350;
            }
        }
        this.screenCount++;
//        log("SCREEN = " + this.screenCount + " of " + this.numOfScreens);
    }

    if (this.horsesFinished > 0) {
        this.raceFinished = true;
    }
};

Horse = function(game, level, xPos, yPos, isPlayer) {

    let imageNum = game.rnd.integerInRange(0, 1)
    let image = "horse" + imageNum;

    Phaser.Sprite.call(this, game, xPos, yPos, image, 0);
    this.game.add.existing(this);
    this.game = game;
    this.level = level;
    this.isPlayer = isPlayer || false;
    this.isPaused = false;
    this.finishingPosition = 0;

    if (this.isPlayer) {
        game.camera.follow(this, Phaser.Camera.FOLLOW_LOCKON);
    }

    this.info = {
        name: 'horse'
    };

    let horseTints = [ 0xffffff ];

    switch(imageNum) {
        case 1:
            horseTints.push(0xdddddd); // light-medium brown
            horseTints.push(0xbbbbbb); // medium brown
            horseTints.push(0x999999); // dark brown
            horseTints.push(0xbb8888); // red brown
            break;
    }


    this.tint = horseTints[game.rnd.integerInRange(0, horseTints.length-1)];

    let saddleTints = [
        0xdf8f12,
//        0x6ade5d,
//        0x5db7de,
//        0x705dde,
//        0xde5daa,
        0xc4c4c4
    ];

    game.physics.arcade.enable(this, Phaser.Physics.ARCADE);
    this.anchor.set(0.5, 1);
    this.body.setSize(
        25, // w
        5,  // h
        3,  // offset_x
        27  // offset_y
    );
//    this.body.collideWorldBounds = true;
//    this.body.immovable = true;

    let saddle = game.add.sprite(-3, -13, 'saddle');
    saddle.tint = saddleTints[game.rnd.integerInRange(0, saddleTints.length-1)];
    this.addChild(saddle);

    let jockey = game.add.sprite(-4, -22, 'jockey' + (!this.isPlayer ? '0' : ''));
    jockey.animations.add('accelerate', [0,1], 7, false).onComplete.add(function(){ jockey.animations.play('run'); }, jockey);
    jockey.animations.add('run', [2,3,4,5,6,7,8,9,10], 20, true);
    this.addChild(jockey);

    let anim = this.animations.add('accelerate', [0,1], 7, false).onComplete.add(function(){ this.animations.play('run'); }, this);
    this.animations.add('run', [2,3,4,5,6,7,8,9,10], 20, true);

    // animation rate
    this.aRate = game.rnd.integerInRange(57, 63) * 0.1;

    this.yTarget = yPos;
    this.currentSpeed = 0;
    this.speed = 120;
    this.animations.play('accelerate');
    jockey.animations.play('accelerate');
    this.jockey = jockey;

    this.text = new RetroText(game, 'font1', 5, 9, -2, -28);
    this.text.alpha = 0;
    this.addChild(this.text);
};
Horse.prototype = Object.create(Phaser.Sprite.prototype);
Horse.prototype.constructor = Horse;
Horse.prototype.pause = function() {

    if (!this.isPaused) {
        this.isPaused = true;
        this.animations.paused = true;
        this.jockey.animations.paused = true;
    }
};
Horse.prototype.resume = function() {

    if (this.isPaused) {
        this.isPaused = false;
        this.animations.paused = false;
        this.jockey.animations.paused = false;
    }
};
Horse.prototype.changeLane = function(val) {

    this.yTarget += val * 10;
};
Horse.prototype.setSpeed = function(sp) {

    let actual = sp;

    if (sp == 10) { sp = 15; }
    this.speed = 120 + sp;

    this.text.setText(actual + "");
    this.text.alpha = 1;
    this.text.y = -28;

    if (this.isPlayer) {
        this.text.tint = 0x5555ff;
    } else {
        this.text.tint = 0xff5555;
    }
};
Horse.prototype.updateAI = function() {

    let speeds = [1,2,3,4,5,5,5,6,6,6,7,7,7,8,9,9,10,10];
    let lanes = [-1,0,0,0,1];

    let s = speeds[game.rnd.integerInRange(0, speeds.length-1)];
    let l = lanes[game.rnd.integerInRange(0, lanes.length-1)];

    this.setSpeed(s);
    this.changeLane(l);
};
Horse.prototype.update = function() {

    if (this.isPaused) {
        this.body.velocity.x = 0;
        this.body.velocity.y = 0;
        return;
    }

    this.text.alpha -= 0.01;
    this.text.y -= 0.1;

//    this.body.velocity.x = this.speed;
    if (this.currentSpeed != this.speed) {
        let dv = this.speed - this.currentSpeed;
        this.currentSpeed += (dv * 0.016666);
        this.animations.getAnimation('run').speed = this.currentSpeed / this.aRate;
        this.jockey.animations.getAnimation('run').speed = this.currentSpeed / this.aRate;
    }
    this.body.velocity.x = this.currentSpeed;// / 0.1666 * game.time.physicsElapsed;

    this.body.velocity.y = (this.yTarget - this.y);

    // make sure the horse doesn't run over the bars
    if (this.y < 69) { this.y = 69; this.yTarget = this.y; }
    if (this.y > 112) { this.y = 112; this.yTarget = this.y; }

    if (this.x >= this.level.finish.x && this.finishingPosition == 0) {
        this.level.horsesFinished++;
        this.finishingPosition = this.level.horsesFinished;
    }
};

Deck = function(game) {

    this.game = game;

    this.defs = conf.entity.cards;

    this.deck = [];

    this.discards = [];

    for (let i = 0 ; i < this.defs.length ; i++) {
        for (let j = 0 ; j < this.defs[i].num ; j++) {
            this.deck.push({
                card: this.defs[i].card,
                index: i
            });
        }
    }

    // recombines the discards with the deck, and shuffles
    this.shuffle = function() {

        // add the discards back to the deck
        if (this.discards.length > 0) {
            this.deck.push.apply(this.deck, this.discards);
            this.discards = [];
        }

        let currentIndex = this.deck.length, temporaryValue, randomIndex;

        // While there remain elements to shuffle...
        while (0 !== currentIndex) {

            // Pick a remaining element...
            randomIndex = Math.floor(Math.random() * currentIndex);
            currentIndex -= 1;

            // And swap it with the current element.
            temporaryValue = this.deck[currentIndex];
            this.deck[currentIndex] = this.deck[randomIndex];
            this.deck[randomIndex] = temporaryValue;
        }
    };

    // returns the next card or FALSE
    this.next = function() {

        // if there's nothing left in the deck then shuffle the discards back in
        if (this.deck.length == 0) {
            this.shuffle();
        }

        // if still no cards then return false (this should never happen)
        if (this.deck.length == 0) {
             return false;
        }

        // return next card
        return this.deck.pop();
    };

    this.discard = function(card) {

        this.discards.push(card);
    }
};

Card = function(game, xPos, yPos, value) {

    let def = null;
    if (typeof(value) == 'number') {
        def = {
            card: conf.entity.cards[value],
            index: value
        };
    } else if (typeof(value) == 'string') {
        for (let i = 0 ; i < conf.entity.cards.length ; i++) {
            if (value == conf.entity.cards[i].card) {
                def = conf.entity.cards[i];
                def = {
                    card: conf.entity.cards[i],
                    index: i
                };
            }
        }
    } else if (typeof(value) == 'object' && typeof(value.index) == 'number' && typeof(value.card) == 'string') {
        def = value;
    }

    if (def == null) {
        log('ERROR: card not recognised');
        log(value);
    }

    Phaser.Sprite.call(this, game, xPos, yPos, 'card', def.index);
    this.game.add.existing(this);
    this.game = game;
    this.fixedToCamera = true;

    this.info = {
        name: 'card',
        def: def
    };

    this.anchor.set(0, 0);

    this.selected = false;

    this.inputEnabled = true;
    this.events.onInputUp.add(function(){
        if (this.info.def.index > 1) {
            this.select();
        }
    }, this);

//    jockey.animations.add('run', [2,3,4,5,6,7,8,9,10], 20, true);
};
Card.prototype = Object.create(Phaser.Sprite.prototype);
Card.prototype.constructor = Card;
Card.prototype.update = function() {};
Card.prototype.getValue = function() {

    if (this.info.def.card.charAt(0) == 'b') {
        return 0;
    } else {
        return parseInt(this.info.def.card.charAt(0));
    }
};
Card.prototype.getDirection = function() {

    switch(this.info.def.card.charAt(1)) {
        case 'u':
            return -1;
            break;
        case 'd':
            return 1;
            break;
        default:
            return 0;
            break;
    }
};
Card.prototype.select = function() {

    if (this.selected) {
        this.selected = false;
        this.tint = 0xffffff;
    } else {
        this.selected = true;
        this.tint = 0xccccff;
    }

};
Card.prototype.set = function(value) {

    let def = null;
    if (typeof(value) == 'number') {
        def = {
            card: conf.entity.cards[value].card,
            index: value
        };
    } else if (typeof(value) == 'string') {
        for (let i = 0 ; i < conf.entity.cards.length ; i++) {
            if (value == conf.entity.cards[i].card) {
                def = {
                    card: conf.entity.cards[i].card,
                    index: i
                };
            }
        }
    } else if (typeof(value) == 'object' && typeof(value.index) == 'number' && typeof(value.card) == 'string') {
        def = value;
    }

    if (def == null) {
        log('ERROR: card not recognised');
        log(value);
    }

    this.info.def = def;
    this.frame = def.index;
};
Card.prototype.get = function() {
    return this.info.def;
};